import os  # pragma: no cover

from django.core.asgi import get_asgi_application  # pragma: no cover

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "helper")  # pragma: no cover

application = get_asgi_application()



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool


class FakePlugin(CMSPluginBase):
    name = "FakePlugin"
    render_template = "fake_plugin.html"


plugin_pool.register_plugin(FakePlugin)



import contextlib
import os
import os.path
import shutil
import sys
import unittest
from copy import copy
from tempfile import mkdtemp
from unittest.mock import patch

import django
from django.test.utils import setup_test_environment, teardown_test_environment

from app_helper import runner
from app_helper.main import _make_settings, core
from app_helper.utils import captured_output, get_user_model, temp_dir, work_in

User = get_user_model()


@contextlib.contextmanager
def wrap_test_environment():
    """
    Create a clean test environment context in which test env can be initialized again.

    Needed by tests in which we run tests in the fake applications invoked by app_helper API.
    """
    try:
        teardown_test_environment()
    except AttributeError:
        pass
    yield
    try:
        setup_test_environment()
    except AttributeError:
        pass


DEFAULT_ARGS = {
    "shell": False,
    "test": False,
    "cms_check": False,
    "compilemessages": False,
    "makemessages": False,
    "makemigrations": False,
    "authors": False,
    "server": False,
    "--xvfb": "",
    "--runner": None,
    "--runner-options": None,
    "--cms": True,
    "--failfast": False,
    "--merge": False,
    "--locale": "",
    "--dry-run": False,
    "--empty": False,
    "--native": False,
    "--persistent": False,
    "--persistent-path": None,
    "--use-daphne": False,
    "--use-channels": False,
    "--bind": "",
    "--port": "",
    "<test-label>": "",
    "<extra-applications>": "",
    "options": "",
    "<command>": "",
}


class CommandTests(unittest.TestCase):
    application = None
    basedir = None
    pofile = None
    mofile = None
    migration_dir = None

    @classmethod
    def setUpClass(cls):
        os.environ.setdefault("DATABASE_URL", "sqlite://localhost/:memory:")
        cls.basedir = os.path.abspath(os.path.join("tests", "test_utils"))
        cls.application = "example1"
        cls.application_2 = "example2"
        with work_in(cls.basedir):
            with captured_output():
                cls.migration_example = os.path.abspath(
                    os.path.join(cls.application, "data", "django_0001_initial.py")
                )
                cls.migration_partial = os.path.abspath(
                    os.path.join(cls.application, "data", "django_0001_partial.py")
                )
                cls.poexample = os.path.abspath(os.path.join(cls.application, "data", "django.po"))
                cls.pofile = os.path.abspath(os.path.join(cls.application, "locale", "en", "LC_MESSAGES", "django.po"))
                cls.mofile = os.path.abspath(os.path.join(cls.application, "locale", "en", "LC_MESSAGES", "django.mo"))
                cls.migration_dir = os.path.abspath(os.path.join(cls.application, "migrations"))
                cls.migration_dir_2 = os.path.abspath(os.path.join(cls.application_2, "migrations"))
                cls.migration_file = os.path.abspath(os.path.join(cls.application, "migrations", "0001_initial.py"))
                cls.migration_file_2 = os.path.abspath(
                    os.path.join(cls.application_2, "migrations", "0001_initial.py")
                )
        try:
            import cms  # noqa: F401
        except ImportError:
            DEFAULT_ARGS["--cms"] = False

    def _clean_modules_directories(self):
        """
        Cleanup all the leftover of migration and po files tests.
        Executed in setup and teardown for simplicity.
        """
        try:
            os.unlink(self.pofile)
        except (OSError, TypeError):
            pass
        try:
            os.unlink(self.mofile)
        except (OSError, TypeError):
            pass
        try:
            if self.migration_dir:
                shutil.rmtree(self.migration_dir)
        except (OSError, TypeError):
            pass
        try:
            if self.migration_dir_2:
                shutil.rmtree(self.migration_dir_2)
        except (OSError, TypeError):
            pass
        try:
            del sys.modules["example1.migrations"]
        except KeyError:
            pass
        try:
            del sys.modules["example2.migrations"]
        except KeyError:
            pass
        try:
            del sys.modules["tests.test_utils.example1.migrations"]
        except KeyError:
            pass
        try:
            del sys.modules["tests.test_utils.example1.migrations.0001_initial"]
        except KeyError:
            pass
        try:
            del sys.modules["tests.test_utils.example2.migrations"]
        except KeyError:
            pass

    def setUp(self):
        self._clean_modules_directories()

    def tearDown(self):
        os.environ["AUTH_USER_MODEL"] = "auth.User"
        self._clean_modules_directories()

    def test_map_argv(self):
        """CLI arguments are mapped to internal options."""
        from app_helper.main import _map_argv

        argv_1 = [
            "helper.py",
            "example1",
            "test",
            "--cms",
            "--persistent-path=test",
            "--extra-settings==something.py",
            "--runner==something.py",
        ]
        target_1 = {
            "--bind": "127.0.0.1",
            "--cms": True,
            "--dry-run": False,
            "--empty": False,
            "--extra-settings": "",
            "--failfast": False,
            "--help": False,
            "--locale": None,
            "--merge": False,
            "--migrate": False,
            "--native": False,
            "--no-migrate": False,
            "--persistent": True,
            "--persistent-path": "test",
            "--port": "8000",
            "--runner": "",
            "--runner-options": None,
            "--verbose": None,
            "--version": False,
            "--xvfb": False,
            "--use-daphne": False,
            "--use-channels": False,
            "<application>": "example1",
            "<command>": None,
            "<extra-applications>": [],
            "<test-label>": [],
            "authors": False,
            "cms_check": False,
            "compilemessages": False,
            "makemessages": False,
            "makemigrations": False,
            "options": [
                "helper.py",
                "test",
                "--cms",
                "--persistent-path=test",
                "--extra-settings==something.py",
                "--runner==something.py",
            ],
            "server": False,
            "setup": False,
            "test": True,
        }
        argv_2 = [
            "helper.py",
            "example1",
            "server",
            "--cms",
            "--persistent-path=test",
            "--extra-settings==something.py",
            "--runner==something.py",
        ]
        target_2 = {
            "--bind": "127.0.0.1",
            "--cms": True,
            "--dry-run": False,
            "--empty": False,
            "--extra-settings": "",
            "--failfast": False,
            "--help": False,
            "--locale": None,
            "--merge": False,
            "--migrate": False,
            "--native": False,
            "--no-migrate": False,
            "--persistent": True,
            "--persistent-path": "test",
            "--port": "8000",
            "--runner": "",
            "--runner-options": None,
            "--verbose": None,
            "--version": False,
            "--xvfb": False,
            "--use-daphne": False,
            "--use-channels": False,
            "<application>": "example1",
            "<command>": None,
            "<extra-applications>": [],
            "<test-label>": [],
            "authors": False,
            "cms_check": False,
            "compilemessages": False,
            "makemessages": False,
            "makemigrations": False,
            "options": [
                "helper.py",
                "server",
                "--cms",
                "--persistent-path=test",
                "--extra-settings==something.py",
                "--runner==something.py",
            ],
            "server": True,
            "setup": False,
            "test": False,
        }

        argv_3 = [
            "helper.py",
            "example1",
            "some_command",
            "--cms",
            "--persistent-path=test",
            "--extra-settings==something.py",
            "--runner==something.py",
        ]
        target_3 = {
            "--bind": "127.0.0.1",
            "--cms": True,
            "--dry-run": False,
            "--empty": False,
            "--extra-settings": "",
            "--failfast": False,
            "--help": False,
            "--locale": None,
            "--merge": False,
            "--migrate": False,
            "--native": False,
            "--no-migrate": False,
            "--persistent": True,
            "--persistent-path": "test",
            "--port": "8000",
            "--runner": "",
            "--runner-options": None,
            "--verbose": None,
            "--version": False,
            "--xvfb": False,
            "--use-daphne": False,
            "--use-channels": False,
            "<application>": "example1",
            "<command>": "some_command",
            "<extra-applications>": [],
            "<test-label>": [],
            "authors": False,
            "cms_check": False,
            "compilemessages": False,
            "makemessages": False,
            "makemigrations": False,
            "options": [
                "helper.py",
                "some_command",
                "--cms",
                "--persistent-path=test",
                "--extra-settings==something.py",
                "--runner==something.py",
            ],
            "server": False,
            "setup": False,
            "test": False,
        }

        application_module = __import__(argv_1[1])
        args = _map_argv(argv_1, application_module)
        self.assertEqual(target_1, args)

        application_module = __import__(argv_2[1])
        args = _map_argv(argv_2, application_module)
        self.assertEqual(target_2, args)

        application_module = __import__(argv_3[1])
        args = _map_argv(argv_3, application_module)
        self.assertEqual(target_3, args)

    def test_extra_settings(self):
        """Settings declared in helper file are merged in default settings."""
        from django.conf import settings

        with work_in(self.basedir):
            with captured_output():
                args = copy(DEFAULT_ARGS)
                with temp_dir() as STATIC_ROOT:  # noqa: N806
                    with temp_dir() as MEDIA_ROOT:  # noqa: N806
                        local_settings = _make_settings(args, self.application, settings, STATIC_ROOT, MEDIA_ROOT)
                        # Testing that helper.py in custom project is loaded
                        self.assertEqual(local_settings.TIME_ZONE, "Europe/Rome")

                        args["--extra-settings"] = "cms_helper_extra.py"
                        local_settings = _make_settings(args, self.application, settings, STATIC_ROOT, MEDIA_ROOT)
                        # Testing that helper.py in the command option is loaded
                        self.assertEqual(local_settings.TIME_ZONE, "Europe/Paris")
                        # Existing application is kept
                        self.assertTrue("app_helper.test_data" in local_settings.INSTALLED_APPS)
                        # New ones are added both on top and in random positions
                        self.assertEqual("djangocms_admin_style", local_settings.INSTALLED_APPS[0])
                        self.assertTrue("some_app" in local_settings.INSTALLED_APPS)

                        self.assertTrue(
                            "django.contrib.sessions.middleware.SessionMiddleware" in local_settings.MIDDLEWARE
                        )
                        self.assertEqual("top_middleware", local_settings.MIDDLEWARE[0])
                        self.assertTrue("some_middleware" in local_settings.MIDDLEWARE)

                        # Check the loaders
                        self.assertTrue(
                            "django.template.loaders.app_directories.Loader"
                            in local_settings.TEMPLATES[0]["OPTIONS"]["loaders"]
                        )
                        # Loaders declared in settings
                        self.assertTrue(
                            "admin_tools.template_loaders.Loader" in local_settings.TEMPLATES[0]["OPTIONS"]["loaders"]
                        )
                        # Existing application is kept
                        self.assertTrue(
                            "django.template.context_processors.request"
                            in local_settings.TEMPLATES[0]["OPTIONS"]["context_processors"]
                        )
                        # New one is added
                        self.assertTrue(
                            "django.template.context_processors.debug"
                            in local_settings.TEMPLATES[0]["OPTIONS"]["context_processors"]
                        )
                        # Check template dirs
                        self.assertTrue("some/dir" in local_settings.TEMPLATES[0]["DIRS"])

    @patch("app_helper.server.autoreload.run_with_reloader")
    def test_server_django(self, run_with_reloader):
        """Run server command and create default user - django version."""
        with work_in(self.basedir):
            User = get_user_model()
            User.objects.all().delete()
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["server"] = True
                core(args, self.application)
            self.assertTrue("A admin user (username: admin, password: admin) has been created." in out.getvalue())
            self.assertEqual(run_with_reloader.call_args[0][0].__module__, "django.core.management.commands.runserver")
        User.objects.all().delete()

    @patch("app_helper.server.autoreload.run_with_reloader")
    def test_server_channels(self, run_with_reloader):
        """Run server command and create default user - channels version."""
        try:
            import channels  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("channels is not available, skipping test")
        if channels.__version__ > "4.0":
            raise unittest.SkipTest("channels 4 removed support for runserver, use daphne version instead, skipping")
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["server"] = True
                args["--use-channels"] = True
                core(args, self.application)
            self.assertTrue("A admin user (username: admin, password: admin) has been created." in out.getvalue())
            self.assertEqual(run_with_reloader.call_args[0][0].__module__, "channels.management.commands.runserver")
        User.objects.all().delete()

    @unittest.skipIf(django.VERSION < (3, 0), "Daphne support requires Django 3.0+")
    @patch("app_helper.server.autoreload.run_with_reloader")
    def test_server_daphne(self, run_with_reloader):
        """Run server command and create default user - daphne version."""
        try:
            import daphne  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("daphne is not available, skipping test")
        with work_in(self.basedir):
            User = get_user_model()
            User.objects.all().delete()
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["server"] = True
                args["--use-daphne"] = True
                core(args, self.application)
            self.assertTrue("A admin user (username: admin, password: admin) has been created." in out.getvalue())
            self.assertEqual(run_with_reloader.call_args[0][0].__module__, "daphne.cli")
        User.objects.all().delete()

    def test_makemigrations(self):
        """Run makemigrations command."""
        with captured_output() as (out, err):
            args = copy(DEFAULT_ARGS)
            args["makemigrations"] = True
            args["<extra-applications>"] = ["example2"]
            core(args, self.application)
        self.assertTrue(os.path.exists(self.migration_file))
        self.assertTrue(os.path.exists(self.migration_file_2))
        self.assertTrue("Create model ExampleModel1" in out.getvalue())
        self.assertTrue("Create model ExampleModel2" in out.getvalue())

    def test_makemigrations_update(self):
        """Update migrations if code changes."""
        os.makedirs(self.migration_dir)
        open(os.path.join(self.migration_dir, "__init__.py"), "w")
        shutil.copy(self.migration_partial, self.migration_file)
        with captured_output() as (out, err):
            args = copy(DEFAULT_ARGS)
            args["makemigrations"] = True
            core(args, self.application)
        self.assertTrue("Migrations for 'example1':" in out.getvalue())

    def test_makemigrations_empty(self):
        """Create empty migrations with makemigrations --empty."""
        with captured_output() as (out, err):
            os.makedirs(self.migration_dir)
            open(os.path.join(self.migration_dir, "__init__.py"), "w")
            shutil.copy(self.migration_example, self.migration_file)
            args = copy(DEFAULT_ARGS)
            args["makemigrations"] = True
            args["--empty"] = True
            core(args, self.application)
        self.assertTrue("Migrations for 'example1':" in out.getvalue())

    def test_makemigrations_merge(self):
        """Run makemigrations --merge."""
        with captured_output() as (out, err):
            args = copy(DEFAULT_ARGS)
            args["makemigrations"] = True
            args["--merge"] = True
            core(args, self.application)
            self.assertTrue("No conflicts detected to merge" in out.getvalue())

    def test_makemessages(self):
        """Run makemessages command runs."""
        with work_in(self.basedir):
            with captured_output():
                args = copy(DEFAULT_ARGS)
                args["makemessages"] = True
                core(args, self.application)
                self.assertTrue(os.path.exists(self.pofile))

    def test_compilemessages(self):
        """Run compilemessages command."""
        with work_in(self.basedir):
            with captured_output():
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["compilemessages"] = True
                core(args, self.application)
                self.assertTrue(os.path.exists(self.mofile))

    def test_cms_check(self):
        """Run cms check command via app-helper."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with captured_output() as (out, err):
            shutil.copy(self.poexample, self.pofile)
            args = copy(DEFAULT_ARGS)
            args["cms_check"] = True
            args["--extra-settings"] = "helper.py"
            args["--migrate"] = False
            core(args, self.application)
        self.assertTrue("Installation okay" in out.getvalue())
        self.assertFalse("[WARNING]" in out.getvalue())
        self.assertFalse("[ERROR]" in out.getvalue())

    def test_cms_check_nocms(self):
        """cms check raise exception if cms is not available."""
        try:
            import cms  # noqa: F401

            raise unittest.SkipTest("django CMS available, skipping test")
        except ImportError:
            pass
        with work_in(self.basedir):
            with self.assertRaises(ImportError):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["cms_check"] = True
                args["--extra-settings"] = "helper.py"
                args["--migrate"] = False
                core(args, self.application)

    def test_any_command_check(self):
        """Run any django command via app-helper."""
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["<command>"] = "check"
                args["options"] = ["helper", "check", "--extra-settings=cms_helper_extra.py"]
                core(args, self.application)
        self.assertTrue("no issues" in out.getvalue())

    def test_any_command_compilemessages(self):
        """Run compilemessages as plain django command."""
        with work_in(os.path.join(self.basedir, self.application)):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["<command>"] = "compilemessages"
                args["options"] = "helper compilemessages --cms -len --verbosity=2".split(" ")
                core(args, self.application)
                self.assertTrue(os.path.exists(self.mofile))

    def test_any_command_migrations(self):
        """Run makemigrations as plain django command."""
        with captured_output() as (out, err):
            args = copy(DEFAULT_ARGS)
            args["<command>"] = "makemigrations"
            args["options"] = "helper makemigrations example2 --verbosity=2".split(" ")
            core(args, self.application)
        self.assertFalse("Create model ExampleModel1" in out.getvalue())
        self.assertFalse(os.path.exists(self.migration_file))
        self.assertTrue("Create model ExampleModel2" in out.getvalue())
        self.assertTrue(os.path.exists(self.migration_file_2))

    def test_testrun(self):
        """Run test via API using custom runner which only picks tests in sample applications."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        path = os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), "data"))
        with wrap_test_environment():
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit_state:
                    args = copy(DEFAULT_ARGS)
                    args["test"] = True
                    args["<application>"] = "example1"
                    args["--persistent"] = True
                    args["--runner"] = "runners.CapturedOutputRunner"
                    args["<test-label>"] = self.application
                    core(args, self.application)
            self.assertTrue("Ran 14 tests in" in err.getvalue())
            self.assertEqual(exit_state.exception.code, 0)
            self.assertTrue(args["STATIC_ROOT"].startswith(path))
            self.assertTrue(args["MEDIA_ROOT"].startswith(path))
            self.assertTrue(os.path.exists(args["STATIC_ROOT"]))
            self.assertTrue(os.path.exists(args["MEDIA_ROOT"]))

    def test_testrun_runner_options(self):
        """Run test with additional options."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with wrap_test_environment():
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit_state:
                    args = copy(DEFAULT_ARGS)
                    args["test"] = True
                    args["<application>"] = "example1"
                    args["--persistent"] = True
                    args["--runner"] = "runners.CapturedOutputRunner"
                    args["<test-label>"] = self.application
                    args["--runner-options"] = "--tag=a-tag"
                    core(args, self.application)
            self.assertTrue("Ran 1 test in" in err.getvalue())
            self.assertEqual(exit_state.exception.code, 0)

    def test_testrun_persistent_path(self):
        """Run test via API using custom runner which only picks tests in sample applications with persistent data."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        path = mkdtemp()
        with wrap_test_environment():
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit_state:
                    args = copy(DEFAULT_ARGS)
                    args["test"] = True
                    args["--persistent"] = True
                    args["--persistent-path"] = path
                    args["--runner"] = "runners.CapturedOutputRunner"
                    args["<test-label>"] = self.application
                    core(args, self.application)
            self.assertTrue("Ran 14 tests in" in err.getvalue())
            self.assertEqual(exit_state.exception.code, 0)
            self.assertTrue(args["STATIC_ROOT"].startswith(path))
            self.assertTrue(args["MEDIA_ROOT"].startswith(path))
            self.assertTrue(os.path.exists(args["STATIC_ROOT"]))
            self.assertTrue(os.path.exists(args["MEDIA_ROOT"]))

    def test_runner_wrong(self):
        """Run non existing test labels via runner file using custom runner."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        if sys.version_info < (3, 5):
            exception = ImportError
        else:
            exception = SystemExit
        with wrap_test_environment():
            with captured_output():
                with self.assertRaises(exception) as exit_state:
                    args = []
                    args.append("helper")
                    args.append("test")
                    args.append("example1")
                    args.append("--runner=runners.CapturedOutputRunner")
                    # passing extra_args is the same as appending to args, except that in real life, args are taken
                    # from sys.argv and extra_args is an argument for developer to use
                    runner.cms("example1", args, extra_args=["whatever"])
        if sys.version_info >= (3, 5):
            self.assertEqual(exit_state.exception.code, 1)

    def test_runner(self):
        """Run tests from helper file."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")

        with wrap_test_environment():
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit_state:
                    args = []
                    args.append("helper")
                    args.append("test")
                    args.append("--extra-settings=helper.py")
                    args.append("example1")
                    runner.cms("example1", args)
            self.assertTrue("visible string" in out.getvalue())
            self.assertFalse("hidden string" in out.getvalue())
            self.assertFalse("hidden string" in err.getvalue())
            self.assertTrue("Ran 14 tests in" in err.getvalue())
            self.assertEqual(exit_state.exception.code, 0)

    def test_runner_compat(self):
        """Run tests from previous helper file default name cms_helper."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")

        with wrap_test_environment():
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit_state:
                    args = []
                    args.append("cms_helper")
                    args.append("test")
                    args.append("example1")
                    runner.cms("example1", args)
        self.assertTrue("visible string" in out.getvalue())
        self.assertFalse("hidden string" in out.getvalue())
        self.assertFalse("hidden string" in err.getvalue())
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit_state.exception.code, 0)

    def test_runner_cms_exception(self):
        """Raise exception if cms-enabled tests are invoked without django CMS."""
        try:
            import cms  # noqa: F401

            raise unittest.SkipTest("django CMS available, skipping test")
        except ImportError:
            pass
        from tests.test_utils.runners import CapturedOutputRunner

        with patch("django.test.runner.DiscoverRunner", CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (__a, __b):
                    with self.assertRaises(ImportError):
                        args = []
                        args.append("helper")
                        runner.cms("example1", args)

    def test_runner_cms_argv(self):
        """Arguments are mapped via cms helper."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")

        def fake_runner(argv):
            return argv

        from tests.test_utils.runners import CapturedOutputRunner

        with patch("django.test.runner.DiscoverRunner", CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    args = []
                    args.append("helper")
                    with patch("app_helper.runner.runner", fake_runner):
                        data = runner.cms("example1", args)
                    self.assertEqual(data, ["helper", "example1", "test", "--cms"])

    def test_runner_argv(self):
        """Arguments are mapped via non-cms helper."""

        def fake_runner(argv):
            return argv

        from tests.test_utils.runners import CapturedOutputRunner

        with patch("django.test.runner.DiscoverRunner", CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    args = []
                    args.append("helper")
                    with patch("app_helper.runner.runner", fake_runner):
                        data = runner.run("example1", args)
                    self.assertEqual(data, ["helper", "example1", "test"])

    def test_setup_cms(self):
        """Settings are set from runner setup."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with captured_output():
            from tests.test_utils import helper

            settings = runner.setup("example1", helper, use_cms=True, extra_args=["--cms"])
        self.assertTrue("example2" in settings.INSTALLED_APPS)
        self.assertTrue("djangocms_text_ckeditor" in settings.INSTALLED_APPS)
        self.assertTrue("sekizai" in settings.INSTALLED_APPS)
        self.assertTrue("cms" in settings.INSTALLED_APPS)

    def test_setup_custom_user(self):
        """Settings are set by cms runner setup."""
        os.environ["AUTH_USER_MODEL"] = "custom_user.CustomUser"
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with captured_output():
            from tests.test_utils import cms_helper_custom

            settings = runner.setup("example1", cms_helper_custom, use_cms=True, extra_args=["--cms"])
        self.assertTrue("example2" in settings.INSTALLED_APPS)
        self.assertTrue("custom_user" in settings.INSTALLED_APPS)
        self.assertTrue("djangocms_text_ckeditor" in settings.INSTALLED_APPS)
        self.assertTrue("sekizai" in settings.INSTALLED_APPS)
        self.assertTrue("cms" in settings.INSTALLED_APPS)
        del os.environ["AUTH_USER_MODEL"]

    def test_setup_nocms(self):
        """Settings are set by non cms runner setup."""
        with captured_output():
            from tests.test_utils import helper

            settings = runner.setup("example1", helper, extra_args=[])
        self.assertTrue("example2" in settings.INSTALLED_APPS)
        self.assertFalse("sekizai" in settings.INSTALLED_APPS)
        self.assertFalse("cms" in settings.INSTALLED_APPS)

    def test_testrun_nocms(self):
        with wrap_test_environment():
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    with self.assertRaises(SystemExit) as exit_state:
                        args = copy(DEFAULT_ARGS)
                        args["test"] = True
                        args["--cms"] = False
                        args["--runner"] = "runners.CapturedOutputRunner"
                        core(args, self.application)
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit_state.exception.code, 0)

    def test_runner_nocms(self):
        """Run tests via non cms runner."""
        with wrap_test_environment():
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    with self.assertRaises(SystemExit) as exit_state:
                        args = []
                        args.append("helper")
                        args.append("example1")
                        args.append("test")
                        args.append("--extra-settings=helper.py")
                        runner.run("example1", args, extra_args=["--runner=runners.CapturedOutputRunner"])
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit_state.exception.code, 0)

    def test_testrun_native(self):
        """Run tests via api with extra settings file."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with wrap_test_environment():
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    args = copy(DEFAULT_ARGS)
                    args["<command>"] = None
                    args["test"] = True
                    args["--cms"] = False
                    args["--native"] = True
                    args["--extra-settings"] = "cms_helper_extra_runner.py"
                    try:
                        core(args, self.application)
                    except SystemExit:
                        pass
        self.assertTrue("Ran 14 tests in" in err.getvalue())

    def test_testrun_pytest(self):
        """Run tests via pytest via API."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with wrap_test_environment():
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["<command>"] = None
                args["test"] = True
                args["--cms"] = False
                args["--runner"] = "app_helper.pytest_runner.PytestTestRunner"
                args["--extra-settings"] = "helper_no_cms.py"
                args["--runner-options"] = "'-k test_create_django_image_object'"
                args["options"] = ["helper", "test", "--failfast", "--verbosity=2"]
                try:
                    core(args, self.application)
                except SystemExit:
                    pass
        self.assertTrue("64 items / 63 deselected / 1 selected" in out.getvalue())
        # warnings will depend on django version and adds too much noise
        self.assertTrue("1 passed, 63 deselected" in out.getvalue())

    def test_runner_pytest(self):
        """Run tests via pytest via helper runner."""
        with wrap_test_environment():
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit_state:
                    args = []
                    args.append("helper_no_cms")
                    args.append("example1")
                    args.append("test")
                    args.append("--extra-settings=helper_no_cms.py")
                    args.append("--runner-options='-k test_create_django_image_object'")
                    args.append("--runner=app_helper.pytest_runner.PytestTestRunner")
                    runner.run("example1", args)
            self.assertTrue("64 items / 63 deselected / 1 selected" in out.getvalue())
            # # warnings will depend on django version and adds too much noise
            self.assertTrue("1 passed, 63 deselected" in out.getvalue())
            self.assertEqual(exit_state.exception.code, 0)

    def test_authors(self):
        """Generate authors file."""
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["authors"] = True
                core(args, self.application)
        self.assertTrue("Generating AUTHORS" in out.getvalue())
        self.assertTrue("* Iacopo Spalletti" in out.getvalue())
        self.assertTrue("Authors (" in out.getvalue())

    def test_urls(self):
        """cms urlconf is loaded if django CMS is installed."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        from django.urls import reverse

        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["makemessages"] = True
                core(args, self.application)
                self.assertTrue(reverse("pages-root"))

    def test_urls_nocms(self):
        """cms urlconf is not loaded if django CMS is not installed."""
        from django.urls import NoReverseMatch, reverse

        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["makemessages"] = True
                args["--cms"] = False
                core(args, self.application)
                with self.assertRaises(NoReverseMatch):
                    reverse("pages-root")



import unittest
from tempfile import gettempdir

from app_helper.utils import make_temp_dir


class TestUtils(unittest.TestCase):
    def test_make_temp_dir(self):
        temp1 = make_temp_dir("suff")
        self.assertTrue(temp1.startswith("/dev/shm"))
        self.assertTrue(temp1.endswith("suff"))

    def test_make_temp_dir_not_shm(self):
        temp1 = make_temp_dir("suff", container="/some/random/path")
        self.assertTrue(temp1.startswith(gettempdir()))
        self.assertTrue(temp1.endswith("suff"))



from django.utils import autoreload

from .utils import _create_db, create_user, get_user_model


def _run_django(settings, bind, port, migrate_cmd, verbose):
    """Run channels runserver."""
    from django.core.management.commands import runserver

    _setup_db(migrate_cmd)
    _init_runserver(runserver, bind, port, verbose)


def _run_daphne(settings, bind, port, migrate_cmd, verbose):
    """Run daphne runserver."""
    from daphne.cli import CommandLineInterface

    _setup_db(migrate_cmd)
    daphne_args = ["-b", bind, "-p", port, "-v", verbose or "1", settings.ASGI_APPLICATION]
    autoreload.run_with_reloader(CommandLineInterface().run, daphne_args)


def _run_channels(settings, bind, port, migrate_cmd, verbose):
    """Run channels runserver."""
    from channels.management.commands import runserver

    _setup_db(migrate_cmd)
    _init_runserver(runserver, bind, port, verbose, channels=True)


def _init_runserver(runserver_module, bind, port, verbose, logger=None, channels=False):
    """Run base django / channels runserver with autoreloader."""
    rs = runserver_module.Command()
    rs.use_ipv6 = False
    rs._raw_ipv6 = False
    rs.addr = bind
    rs.port = port
    if logger:  # pragma: no cover
        rs.logger = logger
    if channels:
        rs.http_timeout = 60
        rs.websocket_handshake_timeout = 5
    autoreload.run_with_reloader(
        rs.inner_run,
        **{
            "addrport": "{}:{}".format(bind, port),
            "insecure_serving": True,
            "use_static_handler": True,
            "use_threading": True,
            "verbosity": verbose,
            "skip_checks": True,
            "use_reloader": True,
        },
    )


def _setup_db(migrate_cmd):
    """Initialize the database running migrations and creating the default user."""
    _create_db(migrate_cmd)
    User = get_user_model()  # NOQA
    if not User.objects.filter(is_superuser=True).exists():
        usr = create_user("admin", "admin@admin.com", "admin", is_staff=True, is_superuser=True)
        print("")
        print("A admin user (username: %s, password: admin) " "has been created." % usr.get_username())
        print("")


def run(settings, bind, port, migrate_cmd, verbose, use_channels, use_daphne):
    """
    Run runserver command with reloader enabled.

    Currently support channels, daphne and plain django.

    :param settings: Django settings module
    :type settings: Settings
    :param bind: host to bind on
    :type bind: str
    :param port: port to bind on
    :type port: int
    :param migrate_cmd: run migrations when creating the database
    :type migrate_cmd: bool
    :param verbose: verbosity level
    :type verbose: int
    :param use_channels: run channels server
    :type use_channels: bool
    :param use_daphne: run daphne server
    :type use_daphne: bool
    """
    try:
        from channels.management.commands import runserver  # noqa: F401

        channels_enabled = True
    except ImportError:
        channels_enabled = False
    try:
        import daphne  # noqa: F401

        daphne_enabled = True
    except ImportError:
        daphne_enabled = False
    if use_channels and channels_enabled:
        _run_channels(settings, bind, port, migrate_cmd, verbose)
    elif use_daphne and daphne_enabled:
        _run_daphne(settings, bind, port, migrate_cmd, verbose)
    else:
        _run_django(settings, bind, port, migrate_cmd, verbose)



import os
import shlex


class PytestTestRunner:
    """Runs pytest to discover and run tests."""

    def __init__(self, verbosity=1, failfast=False, keepdb=False, **kwargs):
        self.verbosity = verbosity
        self.failfast = failfast
        self.keepdb = keepdb
        self.extra_args = kwargs.pop("pytest_args", "")

    def run_tests(self, test_labels, *args, **kwargs):
        """Run pytest and return the exitcode.

        It translates some of Django's test command option to pytest's.
        """
        import pytest

        self.verbosity = kwargs.get("verbosity", self.verbosity)
        self.failfast = kwargs.get("failfast", self.failfast)
        self.keepdb = kwargs.get("keepdb", self.keepdb)
        argv = shlex.split(os.environ.get("PYTEST_ARGS", ""))
        if self.extra_args:
            argv.extend(shlex.split(self.extra_args))
        if self.verbosity == 0:  # pragma: no cover
            argv.append("--quiet")
        if self.verbosity == 2:  # pragma: no cover
            argv.append("--verbose")
        if self.verbosity == 3:  # pragma: no cover
            argv.append("-vv")
        if self.failfast:  # pragma: no cover
            argv.append("--exitfirst")
        if self.keepdb:  # pragma: no cover
            argv.append("--reuse-db")

        argv.extend(test_labels)
        return pytest.main(argv)



import os
import sys
import unittest

from django.test.runner import DiscoverRunner

from app_helper.utils import work_in


class CapturedOutputRunner(DiscoverRunner):
    """Custom runner to discover tests only in sample applications."""

    def build_suite(self, test_labels=None, extra_tests=None, **kwargs):
        self.top_level = os.path.dirname(__file__)
        with work_in(os.path.dirname(__file__)):
            return super().build_suite(test_labels=test_labels, extra_tests=extra_tests, **kwargs)

    def run_suite(self, suite, **kwargs):
        return unittest.TextTestRunner(verbosity=self.verbosity, failfast=self.failfast, stream=sys.stderr).run(suite)



import inspect
import os.path
import sys


def run(app, argv=sys.argv, extra_args=None):
    """
    Run commands in a plain django environment

    :param app: application
    :param argv: arguments (default to sys.argv)
    :param extra_args: list of extra arguments
    """
    if app not in argv[:2]:
        # app is automatically added if not present
        argv.insert(1, app)
    if len(argv) < 3 and "test" not in argv[:2]:
        # test argument is given if not argument is passed
        argv.insert(2, "test")
    if extra_args:
        argv.extend(extra_args)
    return runner(argv)


def cms(app, argv=sys.argv, extra_args=None):
    """
    Run commands in a django cMS environment

    :param app: application
    :param argv: arguments (default to sys.argv)
    :param extra_args: list of extra arguments
    """
    try:
        import cms  # NOQA  # nopyflakes
    except ImportError:
        print("runner.cms is available only if django CMS is installed")
        raise
    if app not in argv[:2]:
        # app is automatically added if not present
        argv.insert(1, app)
    if len(argv) < 3 and "test" not in argv[:2]:
        # test argument is given if not argument is passed
        argv.insert(2, "test")
    if "--cms" not in argv:
        # this is the cms runner, just add the cms argument
        argv.append("--cms")
    if extra_args:
        argv.extend(extra_args)
    return runner(argv)


def setup(app, helper_module, extra_args=None, use_cms=False):
    """
    Setup the Django / django CMS environment and return the environment settings.

    :param app: application
    :param helper_module: helper module
    :param extra_args: list of extra arguments
    :param use_cms: setup a django CMS environemtn
    :return: Django settings module
    """

    def _pytest_setup(settings, module):
        excluded_settings = {
            "PASSWORD_RESET_TIMEOUT": "PASSWORD_RESET_TIMEOUT_DAYS",
            "DEFAULT_FILE_STORAGE": "STORAGES",
            "STATICFILES_STORAGE": "STORAGES",
        }
        default_settings = {"SECRET_KEY": "secret"}
        for setting in dir(settings):
            if setting.isupper():
                setting_value = getattr(settings, setting)

                # Empty settings value in the original value which must have a default are checked on
                # default_settings dictionary and default is set, if available
                default_value = default_settings.get(setting, None)
                if default_value and not setting_value:
                    setting_value = default_value

                # If two settings exclude each other, we check if the alternate setting is already defined
                # in the settings module and if not, we set the current setting
                alternate_setting = excluded_settings.get(setting, None)
                alternate_setting_value = None
                if alternate_setting:
                    alternate_setting_value = getattr(settings, alternate_setting, None)
                if not alternate_setting_value:
                    setattr(module, setting, setting_value)

    helper = helper_module.__file__
    argv = [os.path.basename(helper), app, "setup", "--extra-settings={}".format(helper)]
    if use_cms:
        argv.append("--cms")
    if extra_args:
        argv.extend(extra_args)
    settings = runner(argv)
    if "pytest_django" in sys.modules:
        _pytest_setup(settings, helper_module)
    return settings


def runner(argv):
    from . import HELPER_FILE
    from .main import main

    # This is a hackish way to get the caller file which is the file
    # which contains the HELPER_SETTINGS
    helper = os.path.abspath(inspect.getframeinfo(inspect.stack()[2][0]).filename)
    # check if extra settings has been passed
    # if not, user the helper file
    extra_settings = any(x.startswith("--extra-settings=") for x in argv)
    if os.path.basename(helper) not in (HELPER_FILE,) and not extra_settings:
        argv.append("--extra-settings=%s" % helper)
    return main(argv)



import os
import unittest

from django.contrib.auth.models import AnonymousUser
from django.test import tag

from app_helper.base_test import BaseTestCase
from app_helper.utils import get_user_model_labels


class FakeTests(BaseTestCase):
    _pages_data = (
        {
            "en": {"title": "Page title", "template": "page.html", "publish": True},
            "fr": {"title": "Titre", "publish": True},
            "it": {"title": "Titolo pagina", "publish": False},
        },
        {
            "en": {
                "title": "Second page",
                "template": "page.html",
                "publish": True,
                "parent": "page-title",
                "apphook": "Example",
            },
            "fr": {"title": "Deuxieme", "publish": True},
            "it": {"title": "Seconda pagina", "publish": False},
        },
    )

    def test_fake(self):
        self.assertTrue(True)

    def test_pages(self):
        from django.conf import settings

        data = self.get_pages_data()
        self.assertEqual(set(data[0].keys()), {"en", "fr", "it"})

        if "cms" in settings.INSTALLED_APPS:
            pages = self.get_pages()
            self.assertEqual(len(pages), 2)

    def test_get(self):
        from django.conf import settings

        if "cms" not in settings.INSTALLED_APPS:
            raise unittest.SkipTest("django CMS not available, skipping test")
        from cms.api import add_plugin

        pages = self.get_pages()
        add_plugin(placeholder=pages[0].placeholders.get(slot="content"), plugin_type="FakePlugin", language="en")
        pages[0].publish("en")
        response = self.client.get("/en/")
        self.assertContains(response, "fake text")
        self.assertContains(response, "body{font-weight: bold;}")
        self.assertContains(response, "Page title")

    def test_create_user(self):
        from django.conf import settings

        if "cms" not in settings.INSTALLED_APPS:
            raise unittest.SkipTest("django CMS not available, skipping test")
        user = self.create_user(
            username="random",
            email="random@example.com",
            password="random",
            is_staff=True,
            base_cms_permissions=True,
            permissions=["add_placeholder"],
        )
        self._login_context = self.login_user_context(user)
        self._login_context.user.has_perm("add_placeholdr")
        self._login_context.user.has_perm("add_text")

    @tag("a-tag")
    def test_login_context(self):
        request = self.get_request(None, "en", path="/en")
        self.assertTrue(request.user, AnonymousUser())

        self._login_context = self.login_user_context(self.user)
        request = self.get_request(None, "en", path="/en")
        self.assertTrue(request.user, self.user)
        self._login_context.__exit__(None, None, None)

        request = self.get_request(None, "en", path="/en")
        self.assertTrue(request.user, AnonymousUser())

        with self.login_user_context(self.user):
            request = self.get_request(None, "en", path="/en")
            self.assertTrue(request.user, self.user)

        request = self.get_request(None, "en", path="/en")
        self.assertTrue(request.user, AnonymousUser())

    def test_requests(self):
        from django.conf import settings

        print("visible string")
        with self.captured_output() as (out, err):
            print("hidden string")
        if "cms" in settings.INSTALLED_APPS:
            pages = self.get_pages()

            request = self.get_request(pages[1], "en")
            self.assertEqual(request.path, "/en/second-page/")
            self.assertEqual(request.META["REQUEST_METHOD"], "GET")
            self.assertEqual(request.current_page, pages[1])

            request = self.get_request(None, "en", path="/cumstom-path/")
            self.assertEqual(request.path, "/cumstom-path/")
            self.assertEqual(request.META["REQUEST_METHOD"], "GET")
            self.assertEqual(request.current_page, None)

            request = self.post_request(pages[1], "en", data={"payload": 1})
            self.assertEqual(request.path, "/en/second-page/")
            self.assertEqual(request.META["REQUEST_METHOD"], "POST")
            self.assertEqual(request.POST.get("payload"), "1")

            request = self.get_page_request(
                pages[1],
                lang="en",
                user=self.user_staff,
                edit=True,
            )
            self.assertEqual(request.path, "/en/second-page/")
            self.assertTrue(request.toolbar)
            self.assertEqual(request.META["REQUEST_METHOD"], "GET")

    def test_get_user_model(self):
        if os.environ.get("AUTH_USER_MODEL", "auth.User"):
            user_orm_label, user_model_label = get_user_model_labels()
            self.assertEqual(user_orm_label, "auth.User")
            self.assertEqual(user_model_label, "auth.user")

    def test_create_image(self):
        from filer.models import Image

        image = self.create_filer_image_object()
        self.assertEqual(image.original_filename, self.image_name)
        self.assertEqual(image.width, 800)
        self.assertEqual(image.height, 600)
        self.assertEqual(Image.objects.count(), 1)

    def test_create_filer_image(self):
        from filer.models import Image

        image = BaseTestCase.create_filer_image(self.user, "random.jpg")
        self.assertEqual(image.original_filename, "random.jpg")
        self.assertEqual(image.width, 800)
        self.assertEqual(image.height, 600)
        self.assertEqual(Image.objects.count(), 1)

    def test_create_django_image_object(self):
        image = self.create_django_image_object()
        self.assertEqual(image.name, self.image_name)

    def test_render_plugin(self):
        from django.conf import settings

        if "cms" not in settings.INSTALLED_APPS:
            raise unittest.SkipTest("django CMS not available, skipping test")

        from cms.api import add_plugin

        sample_text = "\nfake text\nen\nPage title\n\n"
        pages = self.get_pages()
        public = pages[0].get_public_object()
        placeholder = pages[0].placeholders.get(slot="content")
        plugin = add_plugin(placeholder=placeholder, plugin_type="FakePlugin", language="en")
        pages[0].publish("en")
        rendered_2 = self.render_plugin(public, "en", plugin)
        self.assertEqual(rendered_2, sample_text)

    def test_request(self):
        from django.conf import settings

        if "cms" not in settings.INSTALLED_APPS:
            raise unittest.SkipTest("django CMS not available, skipping test")

        pages = self.get_pages()

        request = self.get_request(pages[1], lang="en")
        self.assertIsNone(getattr(request, "toolbar", None))

        request = self.get_page_request(pages[1], user=self.user)
        self.assertIsNotNone(getattr(request, "toolbar", None))

        request = self.get_request(pages[1], "en", use_middlewares=True)
        self.assertIsNotNone(getattr(request, "toolbar", None))
        self.assertIsNotNone(getattr(request, "_messages", None))

        request = self.get_request(pages[1], "en", secure=True)
        self.assertTrue(request.is_secure())

    def test_request_full_middlewares(self):
        # naked request
        request = self.get_request(page=None, lang="en", user=None, path="/en/second-page/", use_middlewares=True)
        self.assertEqual(request.user, AnonymousUser())
        with self.assertRaises(AttributeError):
            self.assertTrue(request.current_page.pk)
        self.assertIsNotNone(getattr(request, "_messages", None))

        # passing user
        request = self.get_request(page=None, lang="en", user=self.user, path="/en/second-page/", use_middlewares=True)
        self.assertEqual(request.user, self.user)
        with self.assertRaises(AttributeError):
            self.assertTrue(request.current_page.pk)
        self.assertIsNotNone(getattr(request, "_messages", None))

        # logged in user
        with self.login_user_context(self.user):
            request = self.get_request(page=None, lang="en", user=None, path="/en/second-page/", use_middlewares=True)
            self.assertEqual(request.user, self.user)
            with self.assertRaises(AttributeError):
                self.assertTrue(request.current_page.pk)

    def test_request_full_middlewares_cms(self):
        from django.conf import settings

        if "cms" not in settings.INSTALLED_APPS:
            raise unittest.SkipTest("django CMS not available, skipping test")

        pages = self.get_pages()
        # naked request, page and language is derived from path
        request = self.get_request(page=None, lang=None, user=None, path="/en/second-page/", use_middlewares=True)
        self.assertEqual(request.user, AnonymousUser())
        self.assertEqual(request.LANGUAGE_CODE, "en")
        self.assertEqual(request.current_page.get_absolute_url(), pages[1].get_absolute_url())
        self.assertIsNotNone(getattr(request, "_messages", None))

        # naked request, page and language is derived from path, user is logged in
        request = self.get_request(page=None, lang=None, user=self.user, path="/en/second-page/", use_middlewares=True)
        self.assertEqual(request.user, self.user)
        self.assertEqual(request.LANGUAGE_CODE, "en")
        self.assertEqual(request.current_page.get_absolute_url(), pages[1].get_absolute_url())
        self.assertIsNotNone(getattr(request, "_messages", None))

        # naked request, path is derived from path
        request = self.get_request(page=pages[1], lang="en", user=self.user, use_middlewares=True)
        self.assertEqual(request.user, self.user)
        self.assertEqual(request.path_info, "/en/second-page/")
        self.assertEqual(request.LANGUAGE_CODE, "en")
        self.assertEqual(request.current_page.get_absolute_url(), pages[1].get_absolute_url())
        self.assertIsNotNone(getattr(request, "_messages", None))

        # logged in user through context manager
        with self.login_user_context(self.user):
            request = self.get_request(page=None, lang=None, user=None, path="/en/second-page/", use_middlewares=True)
            self.assertEqual(request.user, self.user)
            self.assertEqual(request.current_page.get_absolute_url(), pages[1].get_absolute_url())



import os
import sys
import unittest

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.http import SimpleCookie
from django.test import Client, TestCase

from app_helper.base_test import GenericHelpersMixin, RequestTestCaseMixin
from app_helper.utils import create_user


class TestRequestTestCaseMixin(TestCase):
    class RequestTestCase(RequestTestCaseMixin, TestCase):
        client = Client()

    def test_request(self):
        """Base request object is created with cookies, session and language."""
        test_instance = self.RequestTestCase()

        request = test_instance.request("/")

        self.assertTrue(request.session)
        self.assertTrue(isinstance(request.user, AnonymousUser))
        self.assertTrue(isinstance(request.cookies, SimpleCookie))
        self.assertTrue(request.errors)
        self.assertEqual(request.LANGUAGE_CODE, "")
        with self.assertRaises(AttributeError):
            self.assertTrue(request.toolbar)

    def test_post_request(self):
        """POST request object is created with cookies, session and language, csrf checks disabled."""
        test_instance = self.RequestTestCase()

        request = test_instance.request("/", method="post", data={"some": "data"})

        self.assertEqual(request.session._session_key, "session_key")
        self.assertTrue(isinstance(request.user, AnonymousUser))
        self.assertTrue(isinstance(request.cookies, SimpleCookie))
        self.assertTrue(request.errors)
        self.assertTrue(request._dont_enforce_csrf_checks)
        self.assertEqual(request.LANGUAGE_CODE, "")
        with self.assertRaises(AttributeError):
            self.assertTrue(request.toolbar)

    def test_auth_request(self):
        """Request object with authenticated user, user is assigned to request."""

        User = get_user_model()  # noqa: N806
        user = User(username="some")
        test_instance = self.RequestTestCase()

        request = test_instance.request("/", method="post", data={"some": "data"}, user=user)
        self.assertTrue(request.session)
        self.assertEqual(request.user, user)
        self.assertTrue(isinstance(request.cookies, SimpleCookie))
        self.assertTrue(request.errors)
        self.assertTrue(request._dont_enforce_csrf_checks)
        self.assertEqual(request.LANGUAGE_CODE, "")
        with self.assertRaises(AttributeError):
            self.assertTrue(request.toolbar)

    def test_lang_request(self):
        """Request with forced language, language is assigned to LANGUAGE_CODE."""

        test_instance = self.RequestTestCase()

        request = test_instance.request("/", lang="en")
        self.assertEqual(request.LANGUAGE_CODE, "en")

    def test_login_user_context(self):
        """login_user_context force requests to be authenticated."""
        user = create_user("some", "some@testcom", "some")
        test_instance = self.RequestTestCase()
        with self.assertRaises(AssertionError):
            with test_instance.login_user_context(user, password="wrong"):
                test_instance.request("/", lang="en")
        with test_instance.login_user_context(user):
            request = test_instance.request("/", lang="en")
            self.assertEqual(request.user, user)
        request = test_instance.request("/", lang="en")
        self.assertTrue(isinstance(request.user, AnonymousUser))

    def test_apply_middleware(self):
        """Request with applied middlewares,"""
        test_instance = self.RequestTestCase()
        response = test_instance.request("/", lang="en", use_middlewares=True)

        self.assertFalse(response.current_page)
        # messages are not faked by the basic :py:attr:`request` call, we can use it to detect if middlewares are run
        self.assertIsNotNone(response._messages)
        try:
            import cms  # noqa: F401

            # toolbar is also set if django CMS is installed
            self.assertTrue(response.toolbar)
        except ImportError:
            pass

    def test_use_toolbar(self):
        """Request with django CMS toolbar."""
        try:
            import cms  # noqa: F401
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        test_instance = self.RequestTestCase()
        response = test_instance.request("/", lang="en", use_toolbar=True)

        self.assertFalse(response.current_page)
        # messages are not faked by the basic :py:attr:`request` call, we can use it to detect if middlewares are run
        with self.assertRaises(AttributeError):
            self.assertIsNone(response._messages)
        # toolbar is also set
        self.assertTrue(response.toolbar)

    def test_set_page(self):
        """Request with django CMS page."""
        try:
            from cms.api import create_page
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        test_instance = self.RequestTestCase()
        page = create_page(title="a page", language="en", template=settings.CMS_TEMPLATES[0][0])
        response = test_instance.request("/", lang="en", page=page)

        self.assertEqual(response.current_page, page)


class TestGenericHelpersMixin(TestCase):
    class GenericHelper(GenericHelpersMixin, TestCase):
        pass

    def test_reload_model(self):
        """:py:meth:`GenericHelper.reload_model` create a new object, do not patch the existing one."""
        user = create_user("some", "some@testcom", "some")
        test_instance = self.GenericHelper()
        user.random_attr = "random"
        user_new = test_instance.reload_model(user)
        with self.assertRaises(AttributeError):
            self.assertIsNotNone(user_new.random_attr)
        self.assertNotEqual(id(user), id(user_new))

    def test_temp_dir(self):
        """:py:meth:`GenericHelper.temp_dir` provide a temporary directory which is removed on context manager exit."""
        test_instance = self.GenericHelper()
        with test_instance.temp_dir() as temp_path:
            test_file = os.path.join(temp_path, "afile")
            open(test_file, "w")
            self.assertTrue(os.path.exists(test_file))
        self.assertFalse(os.path.exists(test_file))

    def test_captured_output(self):
        """:py:meth:`GenericHelper.captured_output` context manager capture output."""
        test_instance = self.GenericHelper()
        with test_instance.captured_output() as (out, err):
            print("out string")
            print("err string", file=sys.stderr)
        self.assertEqual(out.getvalue(), "out string\n")
        self.assertEqual(err.getvalue(), "err string\n")
